'use strict';

const { object } = require("joi");
const request = require('request');
const effilogger = require('../logger/effilogger');
const pluginCommon = require('./plugin-common');
const throttle = require("express-throttle");
const PLUGIN_ID = 'xml-custom-input';
const xmlparser = require('express-xml-bodyparser');
const tools = require('../responses');

var configServerObj = null;

async function InitModule(configPlugin, configServer) {
    configServerObj = configServer;
    effilogger.effilogger.info(`[${PLUGIN_ID}] input initialized`);
} 

async function sendBackAsyncRespose(configPlugin, responseRestApi, err, response) {
    if( err || response == null || (response instanceof Object) == false || response.ok == false ) {      
        effilogger.effilogger.error(`[${PLUGIN_ID}] cannot print document`);  
        return tools.resp_error(responseRestApi, response!=null?response:{
            code: -1,
            message: 'Unknown processing error'
        });
    }
    else
    {
        effilogger.effilogger.info(`[${PLUGIN_ID}] document printed`);  
        return tools.resp_success(responseRestApi, response);
    }
}

async function DisposeModule()
{

}

function HasRestExtension() {
    return true;
}

const pluginsErrorLogger = (error, request, response, next) => {
    effilogger.effilogger.error( `Plugin error, check your payload: ${error.message}`) ;
    next(error) // calling next middleware
}

function convertXMLToPayload(parsedBody) {
    const payloadData = {
        lines: [],
        extralines: [],
        summary : {
            to: 0
        }
    };


    if( parsedBody.dokumenty.sprzedaz == null || parsedBody.dokumenty.sprzedaz.length != 1 ) {
        return null;
    }

    const sprzedaz = parsedBody.dokumenty.sprzedaz[0];
    sprzedaz.pozycja.forEach(p=>{
        console.log(p);
        const rabatValue = parseFloat(p.rabat?p.rabat[0]:null);
        const detalValue = parseFloat(p.cena_det_poz[0]);
        const vatRateValue = parseFloat(p.kod_vat[0]);


        const discounts = [];
        if( rabatValue != null ) {
            discounts.push({
                type: "line",
                discount: {
                    na: "Rabat",
                    ds: p.nazwa[0],
                    rw: Math.ceil(rabatValue * 100),
                    rt: Math.ceil(detalValue * 100),
                    rd: true,
                    st: false,
                    vtp: vatRateValue.toFixed(2).replace('.',',')
                }
            })
        }

        payloadData.lines.push({
            na: p.nazwa[0],
            il: p.ilo_jme[0],
            vtp: vatRateValue.toFixed(2).replace('.',','),
            pr: Math.ceil(detalValue * 100),
            discounts: discounts
        })
    });

    const summaryDetalValue = parseFloat(sprzedaz.wart_csb[0]);
    payloadData.summary.to = Math.ceil(summaryDetalValue * 100);
    
    payloadData.extralines.push({id:39, na: sprzedaz.nr_dok[0], sw:false, sh: false});
    payloadData.extralines.push({id:13, na: sprzedaz.nr_zam_si[0], sw:false, sh: false});
    
    /*    
    parsedBody.dokumenty.sprzedaz.forEach(xmlPart=>{
        console.log('---------------------------');
        console.log(xmlPart);    
    });*/
    console.log(JSON.stringify(payloadData, true));
    return payloadData;
}

function ProcessRestExtension(configPlugin, apiPluginRoutes, throttle_options) {
    //try {
    apiPluginRoutes.use(xmlparser());
    apiPluginRoutes.use(pluginsErrorLogger);
    apiPluginRoutes.post(`/${PLUGIN_ID}/process/paragon`, throttle(throttle_options), (req, res) => {
    
        const messageReceived = req.rawBody;
        //console.log(req.body);
        effilogger.effilogger.debug(`[${PLUGIN_ID}] Received XML data: ${messageReceived}`);
        
        const rest_path = '/paragon';
        const rest_method = 'POST';
        const parsedBody = req.body;
        //console.log(parsedBody);

        //############### Convert XML to PosnetServer payload ##################
        const payloadData = convertXMLToPayload(parsedBody);

        pluginCommon.processRestAPI(PLUGIN_ID, configPlugin, configServerObj, rest_path, rest_method, payloadData, res, sendBackAsyncRespose);
        
    });
//}catch(errors) {
    //console.log('EEEEEEEEEEEEEEEEEEEEEEEEEe');
//}
}

module.exports.InitModule = InitModule;
module.exports.DisposeModule = DisposeModule;


module.exports.HasRestExtension = HasRestExtension;
module.exports.ProcessRestExtension = ProcessRestExtension;
